import * as React from 'react';
import { connect } from 'react-redux';
import { FontAwesomeIcon } from '@fortawesome/react-fontawesome'
import { find, propEq, propOr } from 'ramda';

import { isNilOrEmpty, noop } from 'treejs-utils';
import { STATUS } from 'types/common';

import { addModal, closeModal } from '../../Modal/actions';
import { IModalIdOption, IModalOptions } from '../../Modal/types';
import { addModalComponent } from '../../Modal/utils/registerModal';
import TextField from '../../TextField';

import { StyledSelectBox } from '../style';
import { IOption } from '../types';
import SelectBoxModal from '../components/SelectBoxModal';


interface IProps {
	closeModal: (name: IModalIdOption) => void;
	name: string;
	onBlur?: (e: React.FocusEvent<HTMLInputElement>) => void;
	onChange?: (option?: IOption) => void;
	openModal: (options: IModalOptions) => void;
	options: IOption[];
	status?: STATUS;
	title: string;
	value?: string;
}

interface IState {
	selectedOption: IOption;
}

class SelectBox extends React.PureComponent<IProps, IState> {
	public static defaultProps = {
		onChange: noop,
	};

	public readonly state = {
		selectedOption: {
			code: null as string,
			name: null as string,
		},
	};

	public componentDidMount(): void {
		if (!isNilOrEmpty(this.props.value)) {
			if (!isNilOrEmpty(this.props.options)) {
				const newValue = find(propEq('code', this.props.value), this.props.options);
				this.setState({
					selectedOption: newValue,
				});
			}
		}
	}

	public componentDidUpdate(prevProps: IProps, prevState: IState): void {
		if (isNilOrEmpty(prevState.selectedOption)) {
			return;
		}

		if (prevProps.value !== this.props.value) {
			if (!isNilOrEmpty(this.props.options)) {
				const newValue = find(propEq('code', this.props.value), this.props.options);
				this.setState({
					selectedOption: newValue,
				});
			}
		}
	}

	private handleBlur = (e: React.FocusEvent<HTMLInputElement>): void => {
		if (this.props.onBlur) {
			this.props.onBlur(e);
		}
	};

	private handleOptionClick = (option: IOption): void => {
		this.setState({
			selectedOption: option,
		});
		this.props.onChange(option);
	};

	private handleOptionRemove = (): void => {
		this.setState({
			selectedOption: null,
		});
		this.props.onChange();
	};

	private openModal = (): void => {
		const {
			name,
			title,
			options,
			closeModal,
		} = this.props;
		const {
			selectedOption,
		} = this.state;

		addModalComponent(
			name,
			<SelectBoxModal
				name={name}
				options={options}
				selectedOption={selectedOption}
				closeModal={closeModal}
				onChange={this.handleOptionClick}
				onRemove={this.handleOptionRemove}
			/>
		);
		this.props.openModal({ name, title, width: 300 });
	};

	public render(): React.ReactElement {
		const {
			name,
			title,
			status,
		} = this.props;

		const {
			selectedOption,
		} = this.state;

		return (
			<StyledSelectBox id={name}>
				<TextField
					name={name}
					title={title}
					rightIcon={<FontAwesomeIcon icon="caret-square-down" />}
					onClick={this.openModal}
					onFocus={this.openModal}
					onBlur={this.handleBlur}
					value={propOr('', 'name', selectedOption)}
					status={status}
					readonly
				/>
			</StyledSelectBox>
		);
	}
}

export default connect(
	null,
	{
		openModal: addModal,
		closeModal,
	},
)(SelectBox);
