import * as React from 'react';

import { isNilOrEmpty } from 'treejs-utils';
import { STATUS } from 'types/common';

import { StyledContainer, StyledTextField, StyledLabel, StyledTextFieldIcon } from './style';

interface IProps {
	defaultValue?: string;
	disabled?: boolean;
	name: string;
	onBlur?: (e: React.FocusEvent<HTMLInputElement>, value: string) => void;
	onChange?: (e: React.ChangeEvent<HTMLInputElement>, value: string) => void;
	onClick?: () => void;
	onFocus?: () => void;
	readonly?: boolean;
	rightIcon?: React.ReactElement;
	status?: STATUS;
	title: string;
	type?: 'text' | 'password';
	value?: string;
}

interface IState {
	value: string;
}

class TextField extends React.PureComponent<IProps, IState> {
	private inputRef = React.createRef<HTMLInputElement>();

	public static defaultProps = {
		type: 'text',
	};

	public componentDidMount(): void {
		if (!isNilOrEmpty(this.props.defaultValue)) {
			this.setState({
				value: this.props.defaultValue
			});
		}
	}

	public componentDidUpdate(prevProps: IProps, prevState: IState): void {
		let newValue = this.state.value;

		if (prevProps.value !== this.props.value) {
			newValue = this.props.value;
		} else if (prevState.value !== this.state.value) {
			newValue = this.state.value;
		} else {
			return;
		}

		this.setState({
			value: newValue,
		});
	}

	public readonly state = {
		isActive: false,
		value: this.props.value,
	};

	private setFocus = (): void => {
		this.inputRef.current.focus();
	};

	private handleClick = (): void => {
		if (this.props.onClick) {
			this.props.onClick();
		}
	};

	private handleFocus = (): void => {
		if (this.props.onFocus) {
			this.props.onFocus();
		}
	};

	private handleBlur = (e: React.FocusEvent<HTMLInputElement>): void => {
		if (this.props.onBlur) {
			this.props.onBlur(e, e.currentTarget.value);
		}
	};

	private handleChange = (e: React.ChangeEvent<HTMLInputElement>): void => {
		if (this.props.disabled) {
			return;
		}

		if (this.props.onChange) {
			this.props.onChange(e, e.currentTarget.value);
		}
		this.setState({
			value: e.currentTarget.value,
		});
	};

	public render(): React.ReactElement {
		const {
			name,
			title,
			type,
			disabled,
			readonly,
			rightIcon,
			status,
		} = this.props;

		return (
			<StyledContainer>
				<StyledLabel htmlFor={name}>{title}</StyledLabel>
				<StyledTextField
					ref={this.inputRef}
					autoComplete="off"
					id={name} // for click on label
					name={name}
					type={type}
					disabled={disabled}
					readOnly={readonly}
					value={this.state.value || ''}
					noValue={isNilOrEmpty(this.state.value)}
					onClick={this.handleClick}
					onFocus={this.handleFocus}
					onBlur={this.handleBlur}
					onChange={this.handleChange}
					status={status}
				/>
				<StyledTextFieldIcon onClick={this.setFocus}>
					{rightIcon}
				</StyledTextFieldIcon>
			</StyledContainer>
		);
	}
}

export default TextField;
